package require base64
package require Img
package require img::dted
package require img::raw

# Red Green Blue Cyan Magenta Yellow Magenta Black White
set colorList {
    "#FF0000" 
    "#00FF00" 
    "#0000FF" 
    "#00FFFF" 
    "#FF00FF" 
    "#FFFF00" 
    "#000000" 
    "#FFFFFF" 
}

set wr 5
set hr 7

set h $hr
set w [expr $wr * [llength $colorList]]

set fmtList [list \
    "img" "bmp"  ".bmp" ""                                      \
    "img" "gif"  ".gif" ""                                      \
    "img" "ico"  ".ico" ""                                      \
    "img" "jpeg" ".jpg" ""                                      \
    "img" "pcx"  ".pcx" ""                                      \
    "img" "png"  ".png" ""                                      \
    "img" "ppm"  ".ppm" ""                                      \
    "img" "raw"  ".raw" "-useheader true -nomap true -nchan 3"  \
    "img" "sgi"  ".rgb" ""                                      \
    "img" "sun"  ".ras" ""                                      \
    "img" "tga"  ".tga" ""                                      \
    "img" "tiff" ".tif" ""                                      \
    "img" "xbm"  ".xbm" ""                                      \
    "img" "xpm"  ".xpm" ""                                      \
    "dpi" "bmp"  ".bmp" "-xresolution $w -yresolution $h"       \
    "dpi" "jpeg" ".jpg" "-xresolution $w -yresolution $h"       \
    "dpi" "pcx"  ".pcx" "-xresolution $w -yresolution $h"       \
    "dpi" "png"  ".png" "-xresolution $w -yresolution $h"       \
    "dpi" "tiff" ".tif" "-xresolution $w -yresolution $h"       \
]

proc DrawBmpRect { img xs ys w h mask } {
    set scanline [list]
    for { set x 0 } { $x < $w } { incr x } {
        for { set y 0 } { $y < $h } { incr y } {
            $img put "#FFFFFF" -to [expr $xs + $x] [expr $ys + $y]
            $img transparency set [expr $xs + $x] [expr $ys + $y] $mask
        }
    }
}

proc DrawPhotoRect { img xs ys w h color } {
    set scanline [list]
    for { set x 0 } { $x < $w } { incr x } {
        lappend scanline $color
    }
    set data [list]
    lappend data $scanline
    for { set y 0 } { $y < $h } { incr y } {
        $img put $data -to $xs [expr $ys + $y]
    }
}

proc WriteBase64File { fileName base64Data { varName "" } } {
    puts "WriteBase64Data $fileName $varName"
    set fp [open $fileName w]
    if { $varName ne "" } {
        puts -nonewline $fp "set $varName \\\n\{"
    }
    puts -nonewline $fp $base64Data
    if { $varName ne "" } {
        puts $fp "\}"
    }
    close $fp
}

proc ReadImgData { fileName } {
    set retVal [catch {open $fileName r} fp]
    if { $retVal == 0 } {
        fconfigure $fp -translation binary
        set imgData [read $fp]
        close $fp
        set base64Data [::base64::encode -maxlen 72 $imgData]
        return $base64Data
    } else {
        error "Could not read $fileName"
    }
}


set photo [image create photo -width $w -height $h]
set bmp   [image create photo -width $w -height $h]

set x 0
set y 0
foreach color $colorList {
    DrawPhotoRect $photo $x $y $wr $hr $color
    set x [expr $x + $wr]
}

set x 0
set y 0
set mask false
foreach color $colorList {
    DrawBmpRect $bmp $x $y $wr $hr $mask
    set x [expr $x + $wr]
    set mask [expr ! $mask]
}

foreach { name fmt ext opt } $fmtList {
    set fileName "testimgs/$name$ext"
    puts "Generate image $fileName"
    if { $fmt eq "xbm" } {
        $bmp write $fileName -format $fmt
    } else {
        $photo write $fileName -format "$fmt $opt"
    }

    if { $fmt eq "xpm" } {
        # TODO: XPM should write as base64, too.
        set base64Data [$photo data -format $fmt]
    } elseif { $fmt eq "xbm" } {
        set base64Data [$bmp data -format $fmt]
    } else {
        set base64Data [ReadImgData $fileName]
    }
    WriteBase64File "$fileName.base64" $base64Data "imgdata"
    WriteBase64File "$fileName.b64"    $base64Data
}

set fileName "testimgs/img2.jpg"
image create photo jpg2 -file "testimgs/img.jpg"
puts "Generate image $fileName"
jpg2 write $fileName -format jpeg
set base64Data [ReadImgData $fileName]
WriteBase64File "$fileName.base64" $base64Data "imgdata2"
WriteBase64File "$fileName.b64"    $base64Data

set fileName "testimgs/img.jp2"
puts "Generate image $fileName"
file copy -force "img.jp2" $fileName
set base64Data [ReadImgData $fileName]
WriteBase64File "$fileName.base64" $base64Data "imgdata"
WriteBase64File "$fileName.b64"    $base64Data

set fileName "testimgs/img.dt0"
puts "Generate image $fileName"
file copy -force "e10n48.dt0" $fileName
set base64Data [ReadImgData $fileName]
WriteBase64File "$fileName.base64" $base64Data "imgdata"
WriteBase64File "$fileName.b64"    $base64Data

set fileName "testimgs/img.fpf"
puts "Generate image $fileName"
file copy -force "example1.fpf" $fileName
set base64Data [ReadImgData $fileName]
WriteBase64File "$fileName.base64" $base64Data "imgdata"
WriteBase64File "$fileName.b64"    $base64Data

set fileName "testimgs/img.svg"
puts "Generate image $fileName"
file copy -force "img.svg" $fileName
set base64Data [ReadImgData $fileName]
WriteBase64File "$fileName.base64" $base64Data "imgdata"
WriteBase64File "$fileName.b64"    $base64Data

exit 0
